/**
 * lights.c  
 *
 * Copyright (C) 2000 Jeffrey Fulmer <jdfulmer@armstrong.com>
 * This file is part of wackyd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */ 
#include <ctype.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/kd.h> 
#include <lights.h>

void
init_console( int *c, long int *s )
{
  int console; 
  long int state;

  console = open("/dev/console", 0);
  state = get_lights( console );

  *c = console;
  *s = state;
  return; 
}

long int
get_lights( int console )
{

  long int light = 0;
  if( ioctl( console, KDGETLED, &light ) < 0 ) exit( 1 );
  light = light&7;
 
  return light;
}

int
restore_lights( int console, long int light )
{
  if(( ioctl( console, KDSETLED, light&7)) < 0 ) return -1;
  else return -1; 
}
  
int 
set_lights(int console, wackyd_led_change_t state)
{ 
  int ret;
  long int light = 0;
    
  light = get_lights( console );
  
  switch( state.num ){
    case WACKYD_LED_TURN_ON:  light |= LED_NUM; break;
    case WACKYD_LED_TURN_OFF: if( light & LED_NUM ) light -= LED_NUM; break;
    default:  break;
  }
  switch( state.cap ){
    case WACKYD_LED_TURN_ON:  light |= LED_CAP; break;
    case WACKYD_LED_TURN_OFF: if (light & LED_CAP) light -= LED_CAP; break;
    default:  break;
  }
  switch( state.scr ){
    case WACKYD_LED_TURN_ON:  light |= LED_SCR; break;
    case WACKYD_LED_TURN_OFF: if (light & LED_SCR) light -= LED_SCR; break;
    default:  break;
  }

  if(( ioctl( console, KDSETLED, light&7)) < 0 )
  {
     exit( 1 );
     return 1; /* Keep compiler happy */
  } else {
     if (state.sleep > 0)
         usleep( state.sleep);
     return 0;
  }
}

void 
render_pattern( int console, wackyd_led_change_t pattern[], int elements ) 
{ 
  int i=0;
  for( i = 0; i < elements; i++ ){
    set_lights( console, pattern[i] );
  }
}

