/**
 * Authorized Host List
 *
 * Copyright (C) 2002 Jeffrey Fulmer <jdfulmer@armstrong.com>
 * This file is part of Wackyd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * --
 *
 */ 
#include <list.h>
#include <stdlib.h>
#include <string.h>
#include <wegex.h>
#include <joedog/joedog.h>

/**
 * Eeek!! If this application wants to rely 
 * on threading in the future, this must change.
 */
struct NODE *head = NULL;

int host_compare( char *host, char *match ); 

/**
 * insert values into list
 */
void
add_node( char *hostname )
{
  struct NODE *tail = NULL;
  tail = (struct NODE*)xmalloc( sizeof( struct NODE ));
  tail->hostname = strdup( hostname );
  tail->next = head;
  head = tail;
}

/**
 * delete NODE whose value is *hostname
 */
void
delete_node( char *hostname )
{
  struct NODE *tail = NULL;
  struct NODE *p, *prev;
  p = head;
  if( strncmp(head->hostname, hostname, strlen( hostname )) == 0 ){
    head = head->next;
    free(p); 
  }
  if ( head == NULL ){ 
      tail = NULL;
      return;
  }
  prev = head;
  p = head->next;
  while ( p->next != NULL ){
    if( strncmp( p->hostname, hostname, strlen( hostname )) == 0 ){
      prev->next = p->next;
      free(p);
      return;
    }
    p = p->next;
    prev = prev->next;
  }
  
  if( p->hostname == hostname ){
    prev->next = NULL;
    tail = prev;
    free(p);
  }
}

struct NODE *
find_node( char *hostname ) 
{
  struct NODE *p;
  p = head;
  while( p != NULL ){
    if ( p->hostname == hostname ){
      printf( "%s\n", p->hostname );
    }
    else{
      return NULL;  
    }
    p = p->next;
  }
  return p;
}
  
 
/**
 * Helper util, displays the contents of head
 */ 
void
display_list()
{
  int x = 0;
  struct NODE *p = head;
  while( p != NULL ){
    if( x == 0 )
      printf("authorized clients:    %s\n", p->hostname);
    else
      printf("                       %s\n", p->hostname);
    x++;
    p = p->next;
  }
} 

int 
host_match( char *host, char *addr )
{
  struct NODE *h = head;
  struct NODE *a;

  while( h != NULL ){
    if( host_compare( host, h->hostname ) == 1 )
      return 1;
    h = h->next;
  } 

  a = head;
  while( a != NULL ){
    if( host_compare( addr, a->hostname ) == 1 )
      return 1;
    a = a->next;
  } 
  return 0;
}

int
host_compare( char *host, char *match )
{
  int     res;
  regex_t reg;
  regmatch_t pm[10];
  char pattern[256];
  char ebuf[128];
  const size_t nmatch = 10;

  if( !host  ) return 0;
  if( !match ) return 0;

  if( match[0]=='*' )
    snprintf( pattern, sizeof( pattern ), ".%s", match );
  else
    snprintf( pattern, sizeof( pattern ), "%s", match );
 
  res = regcomp( &reg, pattern, REG_ICASE );
  if( res != 0 ){
    regerror( res, &reg, ebuf, sizeof( ebuf ));
    joe_error( "%s: pattern '%s'\n", ebuf, pattern );
    return -1;
  }
  res = regexec( &reg, host, nmatch, pm, 0 );
  regfree(&reg);
  if( res == REG_NOMATCH ){
    return 0;
  }
  else{
    return 1;
  }
  return 0;
} 

#if 0

/**
 * main function for testing list.c standalone 
 */
int
main()
{
  add_node("10.12.4.2");
  add_node("hardy.armstrong.net");
  add_node("*.armstrong.com");
  add_node("10.12.12.2");
  if( host_match( "hardy.armstrong.com", "10.3.15.2" ) == 1 )
    printf("whoo hoo!\n");
  else
    printf("d\'oh!\n");

  return 0;
}
#endif
