#ifndef SOCK_H
#define SOCK_H

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h> 

#ifdef  HAVE_SSL
# include <openssl/ssl.h>
# include <openssl/err.h>
# include <openssl/rsa.h>
# include <openssl/crypto.h>
# include <openssl/x509.h>
# include <openssl/pem.h>
#endif/*HAVE_SSL*/

/**
 * enum PROTOCOL
 * tcp/ip protocol
 */
typedef enum
{
  WACKY        = 0,
  WACKYS       = 1,
  UNSUPPORTED  = 2
} PROTOCOL; 

typedef enum
{ 
  CLIENT       = 1,
  LISTENER     = 2,
  BOTH         = 4
} TYPE;
 
typedef enum
{
  S_CONNECTING = 1,
  S_READING    = 2,
  S_WRITING    = 4,
  S_DONE       = 8
} STATUS;  

typedef struct
{
  int      sock;
  STATUS   status;
#ifdef  HAVE_SSL
  SSL      *ssl;
  SSL_CTX  *ctx;
#endif/*HAVE_SSL*/
  fd_set   rset;
  fd_set   wset;
} CONN; 

int     WACKYsocket( char *hostname, char *portname, socklen_t *addrlen ); 
int     WACKYaccept( int sock, struct sockaddr_in *clnt, socklen_t *len );
int     WACKYsocket_check( CONN *C ); 
ssize_t WACKYreadline( CONN *C, char *ptr, size_t len );
void    WACKYclose ( CONN *C );

#endif/*SOCK_H*/
