/**
 * util.c
 *
 * Copyright (C) 2000 Jeffrey Fulmer <jdfulmer@armstrong.com>
 * This file is part of wackyd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <setup.h>
#include <pattern.h>

void
parse_speed( char *p )
{
  int x = 0;
  if( p == NULL ){
    my.speed = MEDIUM;
    return;
  }

  for( ; x < strlen( p ); x ++ )
  switch( TOLOWER( p[x] )){
    case 's':
      my.speed = SLOW;
      return;
    case 'm':
      my.speed = MEDIUM;
      return;
    case 'f':
      my.speed = FAST;
      return;
    default:
      my.speed = MEDIUM; 
      break;
  }
  return; 
}

int
update_pidfile( long mypid, IDSET set )
{
  char pid[16];
  char pidfile[512];
  int fd;

  if( !file_exists( RUNDIR )){ 
    if( !file_exists( VARDIR )){
      if(( mkdir( VARDIR, 0755 )) < 0 )
        perror( "unable to make directory" );
    }
    if(( mkdir( RUNDIR, 0755 )) < 0 ){
      perror( "unable to make directory" );
      return -1;
    }
  }

  snprintf( pidfile, sizeof( pidfile ), "%s", (set==CHILD)?CIDFILE:PIDFILE );
  snprintf( pid, sizeof( pid ), "%d", mypid );

  if(( fd = open( pidfile, O_CREAT | O_WRONLY, 0644 )) < 0 ){
    return -1;
  }
  
  if((fd = open( pidfile, O_WRONLY, 0644 )) < 0 ) {
    joe_error( "Unable to write to file" );
    return -1;
    /** not necessarily fatal **/
  }
  write( fd, pid, strlen( pid ));
  close( fd );
  
  return 0;
}

/**
 * boolean, returns true if the file exists,
 * false if it does not.
 */
int
file_exists( char *file )
{
  int  fd;
  if(( fd = open( file, O_RDONLY )) < 0 ){
    close( fd );
    return FALSE;
  }
  else{
    close( fd );
    return TRUE;
  }
  return FALSE;
}

int
die_hard( int verbose, IDSET set )
{
  switch( set ){
    case CHILD:
      infanticide( verbose );
      break;
    case PARENT:
      suicide( verbose );
      break;
    case ALL:
      infanticide( verbose );
      suicide( verbose );
      break;
  } 
}

int
suicide( int verbose )
{
  FILE  *fp;
  char  buf[8];
  char  pidfile[512];
  pid_t pid;

  if (( fp = fopen(PIDFILE, "r")) == NULL ) {
    if( verbose ){
      printf( "Are you sure wackyd is running?\n" );
      printf( "I can't locate %s\n", PIDFILE );
    }
    return -1;
  }
  
  while(( fgets( buf, sizeof( buf ), fp )) != NULL );
  if(( pid = atoi( buf )) > 100 ){
    if( verbose ) printf( "shutting down wackyd pid[%ld]\n", (long)pid );
    kill( pid, SIGTERM );
  }
  else{
    joe_error( "unable to shutdown wackyd\n" );
  }
  fclose( fp );
  remove( PIDFILE );
  return 0;
}

int
infanticide( int verbose )
{
  FILE  *fp;
  char  buf[8];
  char  pidfile[512];
  pid_t pid;
 
  if (( fp = fopen(CIDFILE, "r")) == NULL ) {
    if( verbose ){
      printf( "Are you sure wackyd is running?\n" );
      printf( "I can't locate %s\n", CIDFILE );
    }
    return -1;
  }
 
  while(( fgets( buf, sizeof( buf ), fp )) != NULL );
  if(( pid = atoi( buf )) > 100 ){
    if( verbose ) printf( "shutting down wackyd pid[%ld]\n", (long)pid );
    kill( pid, SIGTERM );
  }
  else{
    joe_error( "unable to shutdown wackyd\n" );
  }
  fclose( fp );
  remove( CIDFILE );
  return 0;
} 


